/*
 * Copyright (c) 2006, Adam Dunkels
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 ///////////////////////////////////////
 // modified for nonbasic syntax
 //
 // Bruce Land, Cornell Univ, May 2010
 ///////////////////////////////////////

#include <inttypes.h>
#include <avr/io.h>
//#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>
#include <util/delay.h>
#include <avr/eeprom.h>

#include "ubasic.h"
#include "uart.h"

#define begin {
#define end }

// bit twiddling 
#define READ(U, N) (((U) >> (N)) & 1u)
#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))
#define FLIP(U, N) ((void)((U) ^= 1u << (N)))


// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

char pgm_str[1000];


/////////////////////////////////////////////////////////////////
//--- getchar ---
char shell_getchar(void) begin
	//while (~READ(UCSR0A, RXC0)) {} ;
	loop_until_bit_is_set(UCSR0A, RXC0);
	return UDR0 ;
end

//--- putchar ---
void shell_putchar(char c) begin
	//while (~READ(UCSR0A, UDRE0)) {} ;
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = c ;
end
/*---------------------------------------------------------------------------*/
int
main(void)
{
  int in_count;
  char current_char, cmd[6];
  
  uart_init();
  stdout = stdin = stderr = &uart_str;
  fprintf(stdout, "interpreter...\n\r");

  //
  // main loop queries for:
  // commands : load, save, run, list, down
  // load : copy eeprom to ram
  // save : copy ram to eeprom
  // run  ; runs the current program
  // list : displays pgm
  // down : downloads pgm from hyperterm
  while(1) begin
      fprintf(stdout, ">");
  	  fscanf (stdin, "%s", &cmd);

      // save:
	  if (strcmp(cmd, "save")==0) begin
		  // save program string to eeprom
		  in_count=0;
		  while ( pgm_str[in_count] != 0 )  //0 terminates pgm>
		  begin   
		  	// write to eeprom 	
			// eeprom_write_byte  (uint8_t  *__p, uint8_t  __value)
			eeprom_write_byte(in_count, pgm_str[in_count]);
			in_count++;
		  end  
		  // write a zero to terminate
		  eeprom_write_byte  (in_count, 0);  
		  fprintf(stdout, "save complete");
	  end
      
	  // load:
	  if (strcmp(cmd, "load")==0) begin
		  // read a program string from eeprom
		  //  uint8_t 	eeprom_read_byte  (const uint8_t  *__p)
		  in_count=0;
		  while ( eeprom_read_byte(in_count) != 0 )  //<cntl-d terminates pgm>
		  begin   
		  	pgm_str[in_count] = eeprom_read_byte(in_count);
			in_count++;
		  end    
		  pgm_str[in_count] = 0;	//terminate the string
		  fprintf(stdout, "load complete");
	  end

	  // down:
	  if (strcmp(cmd, "down")==0) begin
		  // download a program string
		  in_count=0;
		  while ( (current_char = shell_getchar()) != '@' )  //<cntl-d terminates pgm>
		  begin   
		  	pgm_str[in_count++]=current_char; 
		  end    
		  pgm_str[in_count] = 0;	//terminate the string
		  fprintf(stdout, "download complete");
	  end

	  // run:
	  else if (strcmp(cmd, "run")==0) begin
		  // run the program just entered
		  ubasic_init(pgm_str);
		  do {
		    ubasic_run();
		  } while(!ubasic_finished());
      end

	  // list:
	  else if (strcmp(cmd, "list")==0) begin
		  in_count=0;
		  while (pgm_str[in_count] != 0 )
		  begin   
		  	putchar(pgm_str[in_count]); 
			if (pgm_str[in_count]=='\r')
				putchar('\n') ;
            in_count++;
		  end    
	  end
      
	  else if (strcmp(cmd, "help")==0) begin
	      fprintf(stdout,"down -- get pgm PC file\n\r");
		  fprintf(stdout,"run -- run the pgm\n\r");
		  fprintf(stdout,"list -- display pgm\n\r");
		  fprintf(stdout,"save -- put pgm into eeprom\n\r");
		  fprintf(stdout,"load -- get pgm from eeprom\n\r");    
	  end
	  // no cmd:
	  //else fprintf(stdout, "> unknown cmd\n\r");

  end // while(1)
end // main
/*---------------------------------------------------------------------------*/
