/*
 * Copyright (c) 2006, Adam Dunkels
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 ///////////////////////////////////////
 // modified for nonbasic syntax
 //
 // Bruce Land, Cornell Univ, May 2010
 ///////////////////////////////////////

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <util/delay.h>

#include "ubasic.h"
#include "uart.h"

#define begin {
#define end }

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// buttons on PINB
// leds on PORTC
// USART on D0 and D1 

static const char prog[] =
" \
# test program ; \ 
print 'starting demo'; \
peek 3+32, c ; # look at pinb ;\
while c=255 { \
   print 'press button'; \
   delay(200); \
   peek 3+32, c ; \
} ; \
poke 7+32, 255 ; \
for i= 0 to 50 { \
   peek 3+32, c ; \
   poke 8+32, i ; \
   print 'i=', i ; \
   for j=10 to 13 {print 'i-j=', i-j;}; \
   if c=255 {print 'press button';}; \
   if c<255 { \ 
      print 'button pushed'; \
      delay(500); \
      if c=254 {print 'button0';}; \
   } \
   else { \
	  print 'no button' ; \
      if i%2 {print 'mod 2';}; \
   }; \ 
   delay(500); \
}; \
print 'done' ; \
end; \
" ;


// 10,000 nested loops in 6 seconds 
// (with peek, poke)
// just the poke is 4 seconds
// just loop 2 seconds so 0.2 mSec/loop
// so loop time is 
// adding k=k+1; 8.5 sec, 
// so 2.5/10000=0.25 millseconds for add
/*
static const char program[] = 
"print 'start' ;\
poke 7+32, 255 ; \
for i = 1 to 100 { \
  for j = 1 to 100 { \
   poke 8+32, i ; \
  }; \
}; \
print 'done' ;\
end; \
" ;


static const char program[] = 
"print 'start' ; \
for i = 1 to 10000 { j=i/5; } ; \
print 'end' ; \
end ; \
" ;
*/

/*---------------------------------------------------------------------------*/
int
main(void)
{
  char in_count, current_char;
  uart_init();
  stdout = stdin = stderr = &uart_str;
  //fprintf(stdout, "send program...\n\r");

  // run the program 
  ubasic_init(prog);

  do {
    ubasic_run();
  } while(!ubasic_finished());
  
  
  return 0;
}
/*---------------------------------------------------------------------------*/
